#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Données, approches fonctionnelles - correction
# 
# Correction de l'approche fonctionnelle. Elle s'appuie principalement sur des itérateurs et le module [cytoolz](https://pypi.python.org/pypi/cytoolz).

# In[1]:


get_ipython().run_line_magic('matplotlib', 'inline')
import matplotlib.pyplot as plt
plt.style.use('ggplot')
import pyensae
from jyquickhelper import add_notebook_menu
add_notebook_menu()


# Le notebook utilisera des données issues d'une table de mortalité extraite de [table de mortalité de 1960 à 2010](http://www.data-publica.com/opendata/7098--population-et-conditions-sociales-table-de-mortalite-de-1960-a-2010) qu'on récupère à l'aide de la fonction [table_mortalite_euro_stat](http://www.xavierdupre.fr/app/actuariat_python/helpsphinx/actuariat_python/data/population.html#actuariat_python.data.population.table_mortalite_euro_stat).

# ## Exercice 1 : application aux grandes bases de données
# 
# Imaginons qu'on a une base de données de 10 milliards de lignes. On doit lui appliquer deux traitements : ``f1``, ``f2``. On a deux options possibles :
# 
# * Appliquer la fonction ``f1`` sur tous les éléments, puis appliquer ``f2`` sur tous les éléments transformés par ``f1``.
# * Application la combinaison des générateurs ``f1``, ``f2`` sur chaque ligne de la base de données.
# 
# Que se passe-t-il si on a fait une erreur d'implémentation dans la fonction ``f2`` ?

# In[2]:





# ## Exercice 2 : cytoolz
# 
# La note d'un candidat à un concours de patinage artistique fait la moyenne de trois moyennes parmi cinq, les deux extrêmes n'étant pas prises en compte. Il faut calculer cette somme pour un ensemble de candidats avec [cytoolz](https://pypi.python.org/pypi/cytoolz).

# In[3]:


notes = [dict(nom="A", juge=1, note=8),
        dict(nom="A", juge=2, note=9),
        dict(nom="A", juge=3, note=7),
        dict(nom="A", juge=4, note=4),
        dict(nom="A", juge=5, note=5),
        dict(nom="B", juge=1, note=7),
        dict(nom="B", juge=2, note=4),
        dict(nom="B", juge=3, note=7),
        dict(nom="B", juge=4, note=9),
        dict(nom="B", juge=1, note=10),
        dict(nom="C", juge=2, note=0),
        dict(nom="C", juge=3, note=10),
        dict(nom="C", juge=4, note=8),
        dict(nom="C", juge=5, note=8),        
        dict(nom="C", juge=5, note=8),        
        ]

import pandas
pandas.DataFrame(notes)


# In[4]:


import cytoolz.itertoolz as itz
import cytoolz.dicttoolz as dtz
from functools import reduce
from operator import add


# In[5]:


gr = itz.groupby(lambda d: d["nom"], notes)

def select_note(key_value):
    key, value = key_value
    return key, map(lambda d: d["note"], value)

gr_notes = dtz.itemmap(select_note,  gr)

def enleve_extreme(key_value):
    key, value = key_value
    return key, itz.take(3, itz.drop(1,sorted(value)))

def moyenne(key_value):
    key, value = key_value
    return key, reduce(add, value)/3

no_ext = dtz.itemmap( enleve_extreme, gr_notes)

moy = dtz.itemmap( moyenne, no_ext)
moy


# In[6]: